'==================
'SolitaireChess.BAS v1.1
'==================
'Clone of popular chess puzzle game.
'QB64 code by Dav, APR/2022

'New for v1.1  - screen size now based on users desktop.
'                Should displays the same on every system.
'              - Removed unused image.

'==================================================================
'NOTE:  Select 'Output EXE to souce folder' in IDE before compiling
'==================================================================

'Solitaire Chess is a chess board puzzle for one.
'Goal is to leave only one chess piece on the board.
'Using mouse, capture all if the pieces, leaving only one
'piece remaining on the board.  Every move MUST result
'in a capture.  You must obey chess piece movement rules.

'First select piece to move, then select piece to
'capture.  All moves must capture a piece.

'SPACE resets current level.

'Left/Right Arrow keys navigate up/down levels.

'There are 10 levels to play. Enjoy.

'==================================================================

'define deminsions for chessboard
DIM SHARED row, col, size, squares
row = 4: col = 4: squares = row * col
size = _DESKTOPHEIGHT * .80 / 4


'define squares data, x/y, values...
DIM SHARED sv(squares) '
DIM SHARED sv$(squares) '
DIM SHARED sx(squares), sy(squares) 'top x/y cords of squares
DIM SHARED sx2(squares), sy2(squares) ' bottom x/y cords of squares

'share image files
DIM SHARED board&, pawn&, rook&, bishop&, knight&, queen&, king&

'Set screen size
SCREEN _NEWIMAGE(size * 4, size * 4, 32)
DO: LOOP UNTIL _SCREENEXISTS

'Load media files
board& = _LOADIMAGE("solitairechess-data\board.jpg", 32)
selected& = _LOADIMAGE("solitairechess-data\selected2.jpg", 32)
howtoplay& = _LOADIMAGE("solitairechess-data\howtoplay.png")
pawn& = _LOADIMAGE("solitairechess-data\pawn.png")
rook& = _LOADIMAGE("solitairechess-data\rook.png")
bishop& = _LOADIMAGE("solitairechess-data\bishop.png")
knight& = _LOADIMAGE("solitairechess-data\knight.png")
queen& = _LOADIMAGE("solitairechess-data\queen.png")
king& = _LOADIMAGE("solitairechess-data\king.png")
pegup& = _SNDOPEN("solitairechess-data\pegup.wav")
pegdown& = _SNDOPEN("solitairechess-data\pegdown.wav")
clap& = _SNDOPEN("solitairechess-data\sfx_clap.ogg")
goodjob& = _LOADIMAGE("solitairechess-data\goodjob2.png")
nomovesleft& = _LOADIMAGE("solitairechess-data\nomovesleft.png")

level = 1
maxlevel = 10
firstrun = 1

'======
NEWGAME:
'======


'Show current leven in title bar
titl$ = "Solitaire Chess: Level " + STR$(level) + " of" + STR$(maxlevel)
_TITLE titl$


_PUTIMAGE (0, 0)-(size * 4, size * 4), board&: _AUTODISPLAY

'init board squares values
bc = 1 'counter
FOR r = 1 TO row
    FOR c = 1 TO col
        x = (c * size): y = (r * size)
        sx(bc) = x - size: sx2(bc) = x ' generate x/y values
        sy(bc) = y - size: sy2(bc) = y
        sv(bc) = 0
        sv$(bc) = "" ' empty string means empty
        bc = bc + 1
    NEXT
NEXT

SELECT CASE level
    CASE 1: sv$(2) = "R": sv$(5) = "B": sv$(8) = "R"
        sv$(10) = "P": sv$(16) = "Q"
    CASE 2: sv$(1) = "R": sv$(4) = "B": sv$(6) = "P": sv$(7) = "P"
        sv$(10) = "B": sv$(11) = "H": sv$(16) = "H"
    CASE 3: sv$(4) = "R": sv$(6) = "B": sv$(8) = "B": sv$(9) = "H"
        sv$(11) = "R": sv$(16) = "P"
    CASE 4: sv$(4) = "K": sv$(5) = "R": sv$(6) = "Q": sv$(11) = "B"
        sv$(12) = "H": sv$(13) = "P"
    CASE 5: sv$(1) = "B": sv$(3) = "H": sv$(5) = "R": sv$(6) = "B"
        sv$(9) = "P": sv$(16) = "R"
    CASE 6: sv$(3) = "P": sv$(7) = "B": sv$(9) = "P": sv$(10) = "H"
        sv$(13) = "H": sv$(14) = "K": sv$(15) = "R": sv$(16) = "B"
    CASE 7: sv$(2) = "H": sv$(3) = "P": sv$(6) = "H": sv$(7) = "P": sv$(9) = "B"
        sv$(12) = "B": sv$(13) = "R": sv$(16) = "R"
    CASE 8: sv$(3) = "H": sv$(4) = "B": sv$(5) = "P": sv$(6) = "R": sv$(9) = "B"
        sv$(10) = "P": sv$(15) = "H": sv$(16) = "P"
    CASE 9: sv$(1) = "P": sv$(2) = "R": sv$(6) = "R": sv$(7) = "H": sv$(8) = "B"
        sv$(11) = "P": sv$(12) = "B": sv$(16) = "H"
    CASE 10: sv$(1) = "R": sv$(3) = "Q": sv$(6) = "K"
        sv$(8) = "B": sv$(10) = "H": sv$(13) = "P"
END SELECT

'show how to play help on first run
IF firstrun = 1 THEN
    _PUTIMAGE (size - (size / 2), size - (size / 2))-(size * 3 + (size / 2), size * 3 + (size / 2)), howtoplay&
    'Mouse click continues, or 10 sec pause
    t! = TIMER
    DO: k = _MOUSEINPUT
        IF TIMER > t! + 10 THEN EXIT DO
    LOOP UNTIL _MOUSEBUTTON(1)
    'make sure mouse is up first
    DO: k = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0
    firstrun = 0
END IF


'====
MAIN:
'====

selected = 0

DrawBoard
GOSUB CheckForWin
GOSUB CheckMovesLeft

second:

DO
    t = _MOUSEINPUT

    IF _MOUSEBUTTON(1) THEN
        mx = _MOUSEX: my = _MOUSEY
        'see if user clicked a square
        FOR t = 1 TO squares
            mx = _MOUSEX: my = _MOUSEY
            IF mx >= sx(t) AND mx <= sx2(t) AND my >= sy(t) AND my <= sy2(t) THEN

                'only select up if square has piece
                IF sv$(t) <> "" THEN
                    IF selected = 0 THEN
                        _SNDPLAY pegup&
                        'wait until mouse up
                        DO: p = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0
                        'highlited square first
                        LINE (sx(t), sy(t) - 1)-(sx(t) + size, sy(t) - 1 + size), _RGBA(200, 200, 0, 75), BF
                        selbtn = t 'save 1st selected piece info
                        selbtn$ = sv$(t)
                        'made st selection, go get another
                        selected = 1: GOTO second

                    ELSE

                        IF ValidMove%(selbtn$, selbtn, t) = 1 THEN
                            _SNDPLAY pegdown&

                            sv$(t) = selbtn$
                            sv$(selbtn) = ""
                            'flash square for a moment
                            LINE (sx(t), sy(t) - 1)-(sx(t) + size, sy(t) - 1 + size), _RGBA(200, 200, 0, 75), BF
                            _DELAY .1

                        END IF

                        'wait until mouse up
                        DO: p = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0

                        selected = 0: GOTO MAIN
                    END IF
                ELSE
                    DrawBoard: GOTO MAIN
                END IF

            END IF
        NEXT

        doneamove:
        _PUTIMAGE (0, 0)-(size * 4, size * 4), board&
        DrawBoard
        _DISPLAY

    END IF


    'Check for a keypress
    kk$ = INKEY$
    'space resets board
    IF kk$ = CHR$(32) THEN GOTO NEWGAME
    'Right arrows advance to next level
    IF kk$ = CHR$(0) + CHR$(77) THEN
        level = level + 1
        IF level > maxlevel THEN level = 1
        GOTO NEWGAME
    END IF
    'Left Arrows go back a level
    IF kk$ = CHR$(0) + CHR$(75) THEN
        level = level - 1
        IF level < 1 THEN level = maxlevel
        GOTO NEWGAME
    END IF

LOOP

END

'==========
CheckForWin:

win = 0
FOR i = 1 TO squares
    IF sv$(i) <> "" THEN win = win + 1
NEXT
IF win = 1 THEN
    _SNDPLAY clap&
    _PUTIMAGE (size / 1.5, size / 1.5)-(size * 3.5, size * 1.5), goodjob&
    _DELAY 2.5
    level = level + 1
    IF level > maxlevel THEN level = 1
    GOTO NEWGAME
END IF

RETURN

'==============
CheckMovesLeft:

moves = 0
FOR m = 1 TO squares
    FOR n = 1 TO squares
        IF ValidMove%(sv$(m), m, n) AND sv$(n) <> "" THEN moves = 1
    NEXT
NEXT

IF moves = 0 THEN
    _PUTIMAGE (size / 1.5, size / 1.5)-(size * 3.5, size * 1.5), nomovesleft&
    _DELAY 2.5
    GOTO NEWGAME
END IF

RETURN


SUB DrawBoard
    _DISPLAY
    _PUTIMAGE (0, 0)-(size * 4, size * 4), board&

    FOR num = 1 TO squares
        'If square has a piece...
        IF sv$(num) <> "" THEN
            SELECT CASE sv$(num)
                CASE IS = "P": _PUTIMAGE (sx(num), sy(num))-(sx(num) + size, sy(num) + size), pawn&
                CASE IS = "B": _PUTIMAGE (sx(num), sy(num))-(sx(num) + size, sy(num) + size), bishop&
                CASE IS = "R": _PUTIMAGE (sx(num), sy(num))-(sx(num) + size, sy(num) + size), rook&
                CASE IS = "H": _PUTIMAGE (sx(num), sy(num))-(sx(num) + size, sy(num) + size), knight&
                CASE IS = "Q": _PUTIMAGE (sx(num), sy(num))-(sx(num) + size, sy(num) + size), queen&
                CASE IS = "K": _PUTIMAGE (sx(num), sy(num))-(sx(num) + size, sy(num) + size), king&
            END SELECT
        END IF
    NEXT
    _AUTODISPLAY

END SUB

FUNCTION ValidMove% (a$, a, b)

    v = 0

    'If pawn moving...
    IF a$ = "P" THEN
        SELECT CASE a
            'row 1 -- no valid moves from this row
            'Row 2
            CASE 5: IF b = 1 AND sv$(1) = "" THEN v = 1
                IF b = 2 THEN v = 1
            CASE 6: IF b = 1 OR b = 3 THEN v = 1
                IF b = 2 AND sv$(2) = "" THEN v = 1
            CASE 7: IF b = 2 OR b = 4 THEN v = 1
                IF b = 3 AND sv$(3) = "" THEN v = 1
            CASE 8: IF b = 3 THEN v = 1
                IF b = 4 AND sv$(4) = "" THEN v = 1
                'row 3
            CASE 9: IF b = 5 AND sv$(5) = "" THEN v = 1
                IF b = 6 THEN v = 1
            CASE 10: IF b = 5 OR b = 7 THEN v = 1
                IF b = 6 AND sv$(6) = "" THEN v = 1
            CASE 11: IF b = 6 OR b = 8 THEN v = 1
                IF b = 7 AND sv$(7) = "" THEN v = 1
            CASE 12: IF b = 7 THEN v = 1
                IF b = 8 AND sv$(8) = "" THEN v = 1
                'row 4
            CASE 13: IF b = 9 AND sv$(9) = "" THEN v = 1
                IF b = 5 AND sv$(5) = "" AND sv$(9) = "" THEN v = 1
                IF b = 10 THEN v = 1
            CASE 14: IF b = 9 OR b = 11 THEN v = 1
                IF b = 6 AND sv$(6) = "" AND sv$(10) = "" THEN v = 1
                IF b = 10 AND sv$(10) = "" THEN v = 1
            CASE 15: IF b = 10 OR b = 12 THEN v = 1
                IF b = 11 AND sv$(11) = "" THEN v = 1
                IF b = 7 AND sv$(7) = "" AND sv$(11) = "" THEN v = 1
            CASE 16: IF b = 11 THEN v = 1
                IF b = 12 AND sv$(12) = "" THEN v = 1
                IF b = 8 AND sv$(8) = "" AND sv$(12) = "" THEN v = 1
        END SELECT
    END IF

    'If rook moving (queen borrows this too...)
    IF a$ = "R" OR a$ = "Q" THEN
        SELECT CASE a
            'row 1
            CASE 1: IF b = 2 OR b = 5 THEN v = 1
                IF b = 3 AND sv$(2) = "" THEN v = 1
                IF b = 4 AND sv$(2) = "" AND sv$(3) = "" THEN v = 1
                IF b = 9 AND sv$(5) = "" THEN v = 1
                IF b = 13 AND sv$(9) = "" AND sv$(5) = "" THEN v = 1
            CASE 2: IF b = 1 OR b = 3 OR b = 6 THEN v = 1
                IF b = 4 AND sv$(3) = "" THEN v = 1
                IF b = 10 AND sv$(6) = "" THEN v = 1
                IF b = 14 AND sv$(10) = "" AND sv$(6) = "" THEN v = 1
            CASE 3: IF b = 2 OR b = 4 OR b = 7 THEN v = 1
                IF b = 1 AND sv$(2) = "" THEN v = 1
                IF b = 11 AND sv$(7) = "" THEN v = 1
                IF b = 15 AND sv$(11) = "" AND sv$(7) = "" THEN v = 1
            CASE 4: IF b = 3 OR b = 8 THEN v = 1
                IF b = 2 AND sv$(3) = "" THEN v = 1
                IF b = 1 AND sv$(2) = "" AND sv$(3) = "" THEN v = 1
                IF b = 12 AND sv$(8) = "" THEN v = 1
                IF b = 16 AND sv$(12) = "" AND sv$(8) = "" THEN v = 1
                'row 2
            CASE 5: IF b = 1 OR b = 6 OR b = 9 THEN v = 1
                IF b = 7 AND sv$(6) = "" THEN v = 1
                IF b = 8 AND sv$(6) = "" AND sv$(7) = "" THEN v = 1
                IF b = 13 AND sv$(9) = "" THEN v = 1
            CASE 6
                IF b = 2 OR b = 5 OR b = 7 OR b = 10 THEN v = 1
                IF b = 8 AND sv$(7) = "" THEN v = 1
                IF b = 14 AND sv$(10) = "" THEN v = 1
            CASE 7
                IF b = 3 OR b = 6 OR b = 8 OR b = 11 THEN v = 1
                IF b = 5 AND sv$(6) = "" THEN v = 1
                IF b = 15 AND sv$(11) = "" THEN v = 1
            CASE 8
                IF b = 4 OR b = 7 OR b = 12 THEN v = 1
                IF b = 6 AND sv$(7) = "" THEN v = 1
                IF b = 5 AND sv$(6) = "" AND sv$(7) = "" THEN v = 1
                IF b = 16 AND sv$(12) = "" THEN v = 1
            CASE 9: IF b = 5 OR b = 10 OR b = 13 THEN v = 1
                IF b = 1 AND sv$(5) = "" THEN v = 1
                IF b = 11 AND sv$(10) = "" THEN v = 1
                IF b = 12 AND sv$(11) = "" AND sv$(10) = "" THEN v = 1
            CASE 10
                IF b = 6 OR b = 9 OR b = 11 OR b = 14 THEN v = 1
                IF b = 2 AND sv$(6) = "" THEN v = 1
                IF b = 12 AND sv$(11) = "" THEN v = 1
            CASE 11
                IF b = 7 OR b = 10 OR b = 12 OR b = 15 THEN v = 1
                IF b = 3 AND sv$(7) = "" THEN v = 1
                IF b = 9 AND sv$(10) = "" THEN v = 1
            CASE 12
                IF b = 8 OR b = 11 OR b = 16 THEN v = 1
                IF b = 10 AND sv$(11) = "" THEN v = 1
                IF b = 9 AND sv$(10) = "" AND sv$(11) = "" THEN v = 1
                IF b = 4 AND sv$(8) = "" THEN v = 1
            CASE 13
                IF b = 9 OR b = 14 THEN v = 1
                IF b = 1 AND sv$(5) = "" AND sv$(9) = "" THEN v = 1
                IF b = 5 AND sv$(9) = "" THEN v = 1
                IF b = 15 AND sv$(14) = "" THEN v = 1
                IF b = 16 AND sv$(14) = "" AND sv$(15) = "" THEN v = 1
            CASE 14
                IF b = 13 OR b = 10 OR b = 15 THEN v = 1
                IF b = 2 AND sv$(6) = "" AND sv$(10) = "" THEN v = 1
                IF b = 6 AND sv$(10) = "" THEN v = 1
                IF b = 16 AND sv$(15) = "" THEN v = 1
            CASE 15
                IF b = 14 OR b = 11 OR b = 16 THEN v = 1
                IF b = 13 AND sv$(14) = "" THEN v = 1
                IF b = 7 AND sv$(11) = "" THEN v = 1
                IF b = 3 AND sv$(7) = "" AND sv$(11) = "" THEN v = 1
            CASE 16
                IF b = 12 OR b = 15 THEN v = 1
                IF b = 13 AND sv$(14) = "" AND sv$(15) = "" THEN v = 1
                IF b = 14 AND sv$(15) = "" THEN v = 1
                IF b = 4 AND sv$(8) = "" AND sv$(12) = "" THEN v = 1
                IF b = 8 AND sv$(12) = "" THEN v = 1
        END SELECT
    END IF

    'If bishop is moving (queen borrows this too...)
    IF a$ = "B" OR a$ = "Q" THEN
        SELECT CASE a
            'row 1
            CASE 1: IF b = 6 THEN v = 1
                IF b = 11 AND sv$(6) = "" THEN v = 1
                IF b = 16 AND sv$(11) = "" AND sv$(6) = "" THEN v = 1
            CASE 2: IF b = 5 OR b = 7 THEN v = 1
                IF b = 12 AND sv$(7) = "" THEN v = 1
            CASE 3: IF b = 6 OR b = 8 THEN v = 1
                IF b = 9 AND sv$(6) = "" THEN v = 1
            CASE 4: IF b = 7 THEN v = 1
                IF b = 10 AND sv$(7) = "" THEN v = 1
                IF b = 13 AND sv$(10) = "" AND sv$(7) = "" THEN v = 1
                'row 2
            CASE 5: IF b = 2 OR b = 10 THEN v = 1
                IF b = 15 AND sv$(10) = "" THEN v = 1
            CASE 6
                IF b = 1 OR b = 3 OR b = 9 OR b = 11 THEN v = 1
                IF b = 16 AND sv$(11) = "" THEN v = 1
            CASE 7
                IF b = 2 OR b = 4 OR b = 10 OR b = 12 THEN v = 1
                IF b = 13 AND sv$(10) = "" THEN v = 1
            CASE 8: IF b = 3 OR b = 11 THEN v = 1
                IF b = 14 AND sv$(11) = "" THEN v = 1
                'row 3
            CASE 9: IF b = 6 OR b = 14 THEN v = 1
                IF b = 3 AND sv$(6) = "" THEN v = 1
            CASE 10
                IF b = 5 OR b = 7 OR b = 13 OR b = 15 THEN v = 1
                IF b = 4 AND sv$(7) = "" THEN v = 1
            CASE 11
                IF b = 6 OR b = 8 OR b = 14 OR b = 16 THEN v = 1
                IF b = 1 AND sv$(6) = "" THEN v = 1
            CASE 12: IF b = 7 OR b = 15 THEN v = 1
                IF b = 2 AND sv$(7) = "" THEN v = 1
                'row 4
            CASE 13: IF b = 10 THEN v = 1
                IF b = 7 AND sv$(10) = "" THEN v = 1
                IF b = 4 AND sv$(7) = "" AND sv$(10) = "" THEN v = 1
            CASE 14: IF b = 9 OR b = 11 THEN v = 1
                IF b = 8 AND sv$(11) = "" THEN v = 1
            CASE 15: IF b = 10 OR b = 12 THEN v = 1
                IF b = 5 AND sv$(10) = "" THEN v = 1
            CASE 16: IF b = 11 THEN v = 1
                IF b = 6 AND sv$(11) = "" THEN v = 1
                IF b = 1 AND sv$(6) = "" AND sv$(11) = "" THEN v = 1
        END SELECT
    END IF

    'If knight moving
    IF a$ = "H" THEN
        SELECT CASE a
            'row 1
            CASE 1: IF b = 7 OR b = 10 THEN v = 1
            CASE 2: IF b = 8 OR b = 9 OR b = 11 THEN v = 1
            CASE 3: IF b = 5 OR b = 10 OR b = 12 THEN v = 1
            CASE 4: IF b = 6 OR b = 11 THEN v = 1
                'row 2
            CASE 5: IF b = 3 OR b = 11 OR b = 14 THEN v = 1
            CASE 6: IF b = 4 OR b = 12 OR b = 13 OR b = 15 THEN v = 1
            CASE 7: IF b = 1 OR b = 9 OR b = 14 OR b = 16 THEN v = 1
            CASE 8: IF b = 2 OR b = 10 OR b = 15 THEN v = 1
                'row 3
            CASE 9: IF b = 2 OR b = 7 OR b = 15 THEN v = 1
            CASE 10: IF b = 1 OR b = 3 OR b = 8 OR b = 16 THEN v = 1
            CASE 11: IF b = 2 OR b = 4 OR b = 5 OR b = 13 THEN v = 1
            CASE 12: IF b = 3 OR b = 6 OR b = 14 THEN v = 1
                'row 4
            CASE 13: IF b = 6 OR b = 11 THEN v = 1
            CASE 14: IF b = 5 OR b = 7 OR b = 12 THEN v = 1
            CASE 15: IF b = 6 OR b = 8 OR b = 9 THEN v = 1
            CASE 16: IF b = 7 OR b = 10 THEN v = 1
        END SELECT
    END IF

    'If King is moving (queen borrows this too)
    IF a$ = "K" OR a$ = "Q" THEN
        SELECT CASE a
            'row1
            CASE 1: IF b = 2 OR b = 5 OR b = 6 THEN v = 1
            CASE 2: IF b = 1 OR b = 3 OR b = 5 OR b = 6 OR b = 7 THEN v = 1
            CASE 3: IF b = 2 OR b = 4 OR b = 6 OR b = 7 OR b = 8 THEN v = 1
            CASE 4: IF b = 3 OR b = 7 OR b = 8 THEN v = 1
                'row2
            CASE 5: IF b = 1 OR b = 2 OR b = 6 OR b = 9 OR b = 10 THEN v = 1
            CASE 6: IF b = 1 OR b = 2 OR b = 3 OR b = 5 OR b = 7 OR b = 9 OR b = 10 OR b = 11 THEN v = 1
            CASE 7: IF b = 2 OR b = 3 OR b = 4 OR b = 6 OR b = 8 OR b = 10 OR b = 11 OR b = 12 THEN v = 1
            CASE 8: IF b = 3 OR b = 4 OR b = 7 OR b = 11 OR b = 12 THEN v = 1
                'row 3
            CASE 9: IF b = 5 OR b = 6 OR b = 10 OR b = 13 OR b = 14 THEN v = 1
            CASE 10: IF b = 5 OR b = 6 OR b = 7 OR b = 9 OR b = 11 OR b = 13 OR b = 14 OR b = 15 THEN v = 1
            CASE 11: IF b = 6 OR b = 7 OR b = 8 OR b = 10 OR b = 12 OR b = 14 OR b = 15 OR b = 16 THEN v = 1
            CASE 12: IF b = 7 OR b = 8 OR b = 11 OR b = 15 OR b = 16 THEN v = 1
            CASE 13: IF b = 9 OR b = 10 OR b = 14 THEN v = 1
            CASE 14: IF b = 9 OR b = 10 OR b = 11 OR b = 13 OR b = 15 THEN v = 1
            CASE 15: IF b = 10 OR b = 11 OR b = 12 OR b = 14 OR b = 16 THEN v = 1
            CASE 16: IF b = 11 OR b = 12 OR b = 15 THEN v = 1
        END SELECT
    END IF


    ValidMove% = v
END FUNCTION


